<?php /* _\|/_              vim: set et ts=4 sw=4 sts=4 nowrap fdm=marker:
         (o o)                         
+-----oOO-{_}-OOo--------------------------------------------------------+
| ConPresso - Foolproof Content Management                               |
+------------------------------------------------------------------------+
| Copyright (c) Bartels Schoene GmbH & Co. KG, 48149 Muenster, Germany   |
+------------------------------------------------------------------------+
| This source file is subject to ConPresso license version 2002-11-16    |
| that is bundled with this package in the files LICENSE_EN.TXT or       |
| LICENSE_DE.TXT, and is available through the world-wide-web at         |
|     http://www.conpresso.de/license/LICENSE_EN-2002-11-16.TXT or       |
|     http://www.conpresso.de/license/LICENSE_DE-2002-11-16.TXT          |
|                                                                        |
| If you did not receive a copy of the ConPresso license and are unable  |
| to obtain it through the world-wide-web, please send a note to         |
| <license@conpresso.de> so we can e-mail you a copy immediately.        |
+------------------------------------------------------------------------+
| Authors: ConPresso development and support team <support@conpresso.de> |
+------------------------------------------------------------------------+
  $Id$ */

// modded for mysqli support

if (!defined('CPO')) exit;

    class DB_mysql 
    {
        var $Host     = ""; // mysql-server
        var $Database = ""; // database-name
        var $User     = ""; // user and password to log into the DB
        var $Password = "";

        var $Link_ID  = 0;  // result of mysqli_connect()
        var $Query_ID = 0;  // result of mysqli_query()
        var $Record   = array();  // actual mysqli_fetch_array()-result
        var $Row;           // actual row-number

        var $Errno    = 0;  // error-status
        var $Error    = "";

        // functions

        // display an error-message and stop
        function halt($msg) 
        {
            // check if the database access for logging below failed too.
            if (isset($GLOBALS['databaseError'])) 
                die('<br />' . 'An error occured while we were handling an error. This is bad.' . '<br />' . $msg);
            $GLOBALS['databaseError'] = true;
            
            $t  = '<br />';
            $t .= sprintf('<strong>Database error:</strong><br /> %s <br />', htmlspecialchars($msg, ENT_COMPAT, 'ISO-8859-15'));
            $t .= sprintf('<em>%s (%d)</em><br />',
                htmlspecialchars($this->Error, ENT_COMPAT, 'ISO-8859-15'),
                htmlspecialchars($this->Errno, ENT_COMPAT, 'ISO-8859-15'));
            
            if ($GLOBALS['varDEBUG']<=0 && isset($_SESSION['SID_user']['admin']) && $_SESSION['SID_user']['admin']!=ROLE_ADMIN) {
                $t = 'Sorry, an error occured while processing this request.';
                // translation function __() only exists if no fatal error occured
                if (function_exists('__')) $t = __($t);
            }
            print($t);
            
            logaction('database error', 
                    $this->Error . ' (' . $this->Errno . ')' . 
                    '; Request: ' . $_SERVER["REQUEST_URI"] . 
                    (!empty($_SERVER['HTTP_REFERER']) ? '; Referer:' . $_SERVER['HTTP_REFERER'] : '; No referer') . 
                    ';  ' . $msg); 
            die();
        }

        // connect to the database
        function connect() 
        {
            if (isset($GLOBALS['DB_LINK_ID'])) $this->Link_ID = $GLOBALS['DB_LINK_ID'];
            if (!$this->Link_ID) {
                $this->Link_ID = @mysqli_connect($this->Host, $this->User, $this->Password);
                
                if (!$this->Link_ID) {
                    $this->halt('Database connect failed');
                }

                // ConPresso is a LATIN1-based system
                mysqli_set_charset($this->Link_ID, 'latin1');
                // Set SQL-Mode to NONE because ONLY_FULL_GROUP_BY causes problems
                // https://community.conpresso4.de/viewtopic.php?p=29245#p29245
                mysqli_query($this->Link_ID, "SET sql_mode = ''");

                if (!mysqli_select_db($this->Link_ID, $this->Database)) {
                    $this->halt('cannot use database ' . $this->Database);
                }
                $GLOBALS['DB_LINK_ID'] = $this->Link_ID;
            }
        }

        // perform a query
        function query($Query_String) 
        {
            if ($Query_String=='') return;

            $this->connect();

            // printf("Debug: query = %s<br />\n", $Query_String);

            $this->Query_ID = mysqli_query($this->Link_ID, $Query_String);
            $this->Row   = 0;
            $this->Errno = mysqli_errno($this->Link_ID);
            $this->Error = mysqli_error($this->Link_ID);
            if (!$this->Query_ID) 
            {
                $this->halt("Invalid SQL: ".$Query_String);
            }

            if ($GLOBALS['CPODEBUG'] & DEBUG_LOG_QUERIES) $GLOBALS['CPODEBUGLOG'][] = $Query_String;
            if ($GLOBALS['CPODEBUG'] & DEBUG_COUNT_QUERIES) $GLOBALS['CPODEBUGQUERYCOUNT']++;
            return $this->Query_ID;
        }

        // go to the next Record
        function next_record() 
        {
            $this->Record = mysqli_fetch_array($this->Query_ID);
            $this->Row   += 1;
            $this->Errno = mysqli_errno($this->Link_ID);
            $this->Error = mysqli_error($this->Link_ID);

            $stat = is_array($this->Record);
            if (!$stat) 
            {
                mysqli_free_result($this->Query_ID);
                $this->Query_ID = 0;
            }
            return $stat;
        }

        function next_record_assoc() 
        {
            $this->Record = mysqli_fetch_array($this->Query_ID, MYSQLI_ASSOC);
            $this->Row   += 1;
            $this->Errno = mysqli_errno($this->Link_ID);
            $this->Error = mysqli_error($this->Link_ID);

            $stat = is_array($this->Record);
            if (!$stat) 
            {
                mysqli_free_result($this->Query_ID);
                $this->Query_ID = 0;
            }
            return $stat;
        }

        // go to the Record at position $pos
        function seek($pos) 
        {
            $status = mysqli_data_seek($this->Query_ID, $pos);
            if ($status)
            $this->Row = $pos;
            return;
        }

        function num_rows() 
        {
            return mysqli_num_rows($this->Query_ID);
        }

        function affected_rows() 
        {
            return mysqli_affected_rows($this->Link_ID);
        }

        function num_fields() 
        {
            return mysqli_num_fields($this->Query_ID);
        }

        function v($Name) 
        {
            if (isset($this->Record[$Name])) return $this->Record[$Name];
        }

        function p($Name) 
        {
            if (isset($this->Record[$Name])) print $this->Record[$Name];
        }

        function table_exists($table) 
        { 
            $this->connect(); 
            $tables=mysqli_query($this->Link_ID, "SHOW TABLES"); 
            while (list($temp)=mysqli_fetch_array($tables)) { 
                if($temp == $table) return 1; 
            } 
            return 0; 
        }

        function record() 
        {
            return $this->Record;
        }

        function lastInsertId()
        {
            return mysqli_insert_id($this->Query_ID);
        }
    }
