<?php /* _\|/_              vim: set et ts=4 sw=4 sts=4 nowrap fdm=marker:
         (o o)                         
+-----oOO-{_}-OOo--------------------------------------------------------+
| ConPresso - Foolproof Content Management                               |
+------------------------------------------------------------------------+
| Copyright (c) Bartels Schoene GmbH & Co. KG, 48149 Muenster, Germany   |
+------------------------------------------------------------------------+
| This source file is subject to ConPresso license version 2002-11-16    |
| that is bundled with this package in the files LICENSE_EN.TXT or       |
| LICENSE_DE.TXT, and is available through the world-wide-web at         |
|     http://www.conpresso.de/license/LICENSE_EN-2002-11-16.TXT or       |
|     http://www.conpresso.de/license/LICENSE_DE-2002-11-16.TXT          |
|                                                                        |
| If you did not receive a copy of the ConPresso license and are unable  |
| to obtain it through the world-wide-web, please send a note to         |
| <license@conpresso.de> so we can e-mail you a copy immediately.        |
+------------------------------------------------------------------------+
| Authors: ConPresso development and support team <support@conpresso.de> |
+------------------------------------------------------------------------+
  $Id$ */
if (!defined('CPO')) exit;

$stack  = array(); // contains all start elements we found

$db = new DB;
$droppedTables = array();

$meta     = array();
$colgroup = array();
$tr       = array();
$table    = '';

$fndElement = array(
    'conpresso' => false,
    'data' => false,
    'table' => false,
    'colgroup' => false,
    'col' => false,
    'tr' => false
);

$skipTables = array('_block', '_log', '_log_login', '_sessions');

function foundStartElement($parser, $name, $attrs)
{
    global $fndElement, $stack, $meta, $table, $colgroup, $tr, $skipTables,
           $dbPrefix, $db;

    array_push(
        $stack, 
        array('name' => $name, 'attrs' => $attrs, 'characterData' => '')
    );

    switch ($name) {
    case 'conpresso':
        $fndElement['conpresso'] = true;
        break;

    case 'version':
        if (!$fndElement['conpresso']) break;

        $meta['version'] = '';
        break;

    case 'exporttimestamp':
        if (!$fndElement['conpresso']) break;

        $meta['exporttimestamp'] = '';
        break;

    case 'prefix':
        if (!$fndElement['conpresso']) break;

        $meta['prefix'] = '';
        break;

    case 'data':
        if (!$fndElement['conpresso']) break;

        $fndElement['data'] = true;
        break;

    case 'table':
        if (!$fndElement['conpresso']) break;
        if (!$fndElement['data'])      break;
        if (!array_key_exists('name', $attrs)) break;
        if (in_array($attrs['name'], $skipTables)) break;

        $fndElement['table'] = true;
        $table = $attrs['name'];

        $query = 'DELETE FROM '.$dbPrefix.$table;
        // print(htmlspecialchars($query, ENT_COMPAT, 'ISO-8859-15').'<br />'); flush();
        $db->query($query);
        break;

    case 'colgroup':
        if (!$fndElement['conpresso']) break;
        if (!$fndElement['data'])      break;
        if (!$fndElement['table'])     break;

        $fndElement['colgroup'] = true;
        $colgroup = array();
        break;

    case 'col':
        break;

    case 'tr':
        if (!$fndElement['conpresso']) break;
        if (!$fndElement['data'])      break;
        if (!$fndElement['table'])     break;

        $fndElement['tr'] = true;
        $tr = array();
        break;

    case 'td':
        break;
    }
}

function foundEndElement($parser, $name)
{
    global $fndElement, $stack, $meta, $table, $colgroup, $tr, 
           $dbPrefix, $db;

    $endElement = array_pop($stack);

    switch ($endElement['name']) {
    case 'conpresso':
        $fndElement['conpresso'] = false;
        break;

    case 'version':
        if (!$fndElement['conpresso']) break;

        $meta['version'] = $endElement['characterData'];
        break;

    case 'exporttimestamp':
        if (!$fndElement['conpresso']) break;

        $meta['exporttimestamp'] = $endElement['characterData'];
        break;

    case 'prefix':
        if (!$fndElement['conpresso']) break;

        $meta['prefix'] = $endElement['characterData'];
        break;

    case 'data':
        if (!$fndElement['conpresso']) break;

        $fndElement['data'] = false;
        break;

    case 'table':
        if (!$fndElement['conpresso']) break;
        if (!$fndElement['data'])      break;

        $fndElement['table'] = false;
        break;

    case 'colgroup':
        if (!$fndElement['conpresso']) break;
        if (!$fndElement['data'])      break;
        if (!$fndElement['table'])     break;

        $fndElement['colgroup'] = false;
        break;

    case 'col':
        if (!$fndElement['conpresso']) break;
        if (!$fndElement['data'])      break;
        if (!$fndElement['table'])     break;
        if (!$fndElement['colgroup'])  break;

        $colgroup[] = $endElement['characterData'];
        break;

    case 'tr':
        if (!$fndElement['conpresso']) break;
        if (!$fndElement['data'])      break;
        if (!$fndElement['table'])     break;

        $fndElement['tr'] = false;

        $query = sprintf('INSERT INTO %s%s SET ', utf8_decode($dbPrefix), utf8_decode($table));
        foreach($tr as $key => $value) {
            $query .= sprintf("%s = '%s', ", utf8_decode($key), addslashes(utf8_decode($value))); 
        }
        $query = substr($query, 0, -2); // remove ", " at end
        //print(htmlspecialchars($query, ENT_COMPAT, 'ISO-8859-15').'<br />'); flush();
        $db->query($query);
        break;

    case 'td':
        if (!$fndElement['conpresso']) break;
        if (!$fndElement['data'])      break;
        if (!$fndElement['table'])     break;
        if (!$fndElement['tr'])  break;

        if (!array_key_exists('encoding', $endElement['attrs'])) {
            $endElement['attrs']['encoding'] = '';
        }
        switch ($endElement['attrs']['encoding']) {
        case 'base64':
            $tr[$colgroup[count($tr)]] 
                = base64_decode(trim($endElement['characterData']));
            break;
        default:
            $tr[$colgroup[count($tr)]] = $endElement['characterData'];
            break;
        }
        break;
    }
}

function foundCharacterData($parser, $data)
{
    global $stack;
    
    $stack[count($stack)-1]['characterData'] .= $data;
}

$xml_parser = xml_parser_create();

xml_set_element_handler($xml_parser, 'foundStartElement', 'foundEndElement');
xml_set_character_data_handler($xml_parser, 'foundCharacterData');

// should be disabled to not violate the XML 1.0 specs 
xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, false); 

if (!($fp = fopen($filename, 'r'))) {
   echo ___('Could not open importfile...');
   return;
}

echo ___('Importing data, this might take a while... ')."\n";

while ($data = fread($fp, 4096)) {

    if (xml_parse($xml_parser, $data, feof($fp))) continue;

    printf('XML error: %s at line %d',
        htmlspecialchars(xml_error_string(xml_get_error_code($xml_parser)), ENT_COMPAT, 'ISO-8859-15'),
        htmlspecialchars(xml_get_current_line_number($xml_parser), ENT_COMPAT, 'ISO-8859-15')
    );
}
fclose($fp);
unlink($filename);

echo '<strong>'.__('done.')."</strong>\n";
flush();
logaction('imported data', $_FILES['importfile']['name']);
?>
