<?php /* _\|/_              vim: set et ts=4 sw=4 sts=4 nowrap fdm=marker:
         (o o)                         
+-----oOO-{_}-OOo--------------------------------------------------------+
| ConPresso - Foolproof Content Management                               |
+------------------------------------------------------------------------+
| Copyright (c) Bartels Schoene GmbH & Co. KG, 48149 Muenster, Germany   |
+------------------------------------------------------------------------+
| This source file is subject to ConPresso license version 2002-11-16    |
| that is bundled with this package in the files LICENSE_EN.TXT or       |
| LICENSE_DE.TXT, and is available through the world-wide-web at         |
|     http://www.conpresso.de/license/LICENSE_EN-2002-11-16.TXT or       |
|     http://www.conpresso.de/license/LICENSE_DE-2002-11-16.TXT          |
|                                                                        |
| If you did not receive a copy of the ConPresso license and are unable  |
| to obtain it through the world-wide-web, please send a note to         |
| <license@conpresso.de> so we can e-mail you a copy immediately.        |
+------------------------------------------------------------------------+
| Authors: ConPresso development and support team <support@conpresso.de> |
+------------------------------------------------------------------------+
  $Id$ */
if (!defined('CPO')) exit;

class translate
{
    public $language     = '';
    public $translations = array(array());
    public $files = array();
    public $languageCache = array();

    // constructor
    function __construct($lang = 'deutsch') 
    {
        $this->setLanguage($lang);
    }

    // sets the language and loads the new translations
    function setLanguage($language = 'deutsch')
    {
        $this->language = $language;
        return $this->loadTranslations();
    }

    // adds language files
    function addTranslation($file, $ressource = '')
    {
        if ($ressource=='') $ressource = CPO_BASEDIR.'_include/lang/';
        if (substr($ressource, -1)!='/') $ressource .= '/';

        if (!@file_exists($ressource.$this->language.'/'.$file.'.inc.php') 
            || !@is_readable($ressource.$this->language.'/'.$file.'.inc.php')) 
            return false;

        $this->languageCache = array();

        $this->files[] = array('dir'=>$ressource, 'file'=>$file);

        // add this translation to the translation array
        include($ressource.$this->language.'/'.$file.'.inc.php'); 
        $this->translations = array_merge($this->translations, $translation);
        return true;
    }

    // used to load the translations into the translations-Array
    function loadTranslations($l = '')
    {
        if ($l=='') $l = $this->language;

        if (array_key_exists($l, $this->languageCache)) {
            $this->translations          = $this->languageCache[$l]['translations'];
            if (isset($this->languageCache[$l]['translationDates'])) $GLOBALS['translationDates'] = $this->languageCache[$l]['translationDates'];
            return true;
        }

        $this->translations = array();
        foreach ($this->files as $k=>$v) {
            $file = $v['dir'].$l.'/'.$v['file'].'.inc.php';
            if (!@file_exists($file) || !@is_readable($file)) continue;

            include($file); 
            $this->translations = array_merge($this->translations, $translation);
        }
        $this->languageCache[$l]['translations']     = $this->translations;
        if (isset($GLOBALS['translationDates']))
            $this->languageCache[$l]['translationDates'] = $GLOBALS['translationDates'];

        return true;
    }

    // returns the actual language or FALSE if not set
    function getLanguage()
    {
        return $this->language;
    }

    function getTranslation($text, $lang = '')
    {
        if ($lang!='') $this->loadTranslations($lang);

        if (!is_array($this->translations) || !array_key_exists($text, $this->translations)) {
            $this->missingTranslations($text);
            return $text;
        }

        $t = $this->translations[$text];

        if ($lang!='') $this->loadTranslations($this->getLanguage());

        return $t;
    }

    function getDateTranslation($date, $lang='')
    {
        if ($lang!='') {
            $GLOBALS['translationDates']=array();
            $this->loadTranslations($lang);
        }
        if (!is_array($GLOBALS['translationDates'])) $GLOBALS['translationDates']=array();
        foreach ($GLOBALS['translationDates'] as $k => $v) {
            $date = preg_replace('/\b'.preg_quote($k, '/').'\b/', $v, $date);
        }
        if ($lang!='') {
            $GLOBALS['translationDates']=array();
            $this->loadTranslations($this->getLanguage());
        }

        return $date;
    }

    function findTranslations()
    {
        $tl = array();
        $d = dir(CPO_BASEDIR.'_include/lang');
        while($entry=$d->read()) {
            if ($entry=='CVS') continue; 
            if (!is_dir(CPO_BASEDIR.'_include/lang/'.$entry) || $entry=='.' || $entry=='..') continue;
            $tl[] = $entry;
        }
        $d->close();
        return ($tl);
    }


    function missingTranslations($text)
    {
        global $cpoSystem;

        if (!empty($_SESSION['SID_user']['language']) && $_SESSION['SID_user']['language']!=$cpoSystem['language']) {
            $strLang = $_SESSION['SID_user']['language'];
        } else {
            $strLang = $cpoSystem['language'];
        }
        // either put into logfile (DB)
        if (isset($cpoSystem['missingTranslations']) && $cpoSystem['missingTranslations']=="log") logaction('missing translation ('.$strLang.')', 'Text: '.$text.' / File: '.$_SERVER['PHP_SELF']);
        
        // or put into textfile (FILE) ***************************************************************************
        if (isset($cpoSystem['missingTranslations']) && $cpoSystem['missingTranslations']=="text") file_put_contents( CPO_BASEDIR.'_include/lang/missing_'.$strLang.'.txt', $text.' / File: '.$_SERVER['PHP_SELF']."\n", FILE_APPEND ) ;
    }
}

if (!empty($_SESSION['SID_user']['language']) && $_SESSION['SID_user']['language']!=$cpoSystem['language']) {
    $translator = new translate($_SESSION['SID_user']['language']);
} else {
    $translator = new translate($cpoSystem['language']);
}

if (defined('SETUP') && isset($_POST['language'])) {
    $translator = new translate($_POST['language']);
} elseif (defined('SETUP') && !isset($_POST['language'])) {
    $translator = new translate('deutsch');
}

$translator->addTranslation('common');

function __($text, $lang = '')
{
    global $translator;
    return($translator->getTranslation($text, $lang));
}

function ___($text, $lang = '')
{
    global $translator;
    return (htmlspecialchars($translator->getTranslation($text, $lang), ENT_COMPAT, 'ISO-8859-15'));
}